# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .result_type import ResultType
import datetime as dt
from .parent_biomarker_data import ParentBiomarkerData
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class BiomarkerResult(UniversalBaseModel):
    """
    Represent the schema for an individual biomarker result.
    """

    name: str
    slug: typing.Optional[str] = None
    value: float
    result: str
    type: ResultType
    unit: typing.Optional[str] = None
    timestamp: typing.Optional[dt.datetime] = None
    notes: typing.Optional[str] = None
    min_range_value: typing.Optional[float] = None
    max_range_value: typing.Optional[float] = None
    is_above_max_range: typing.Optional[bool] = None
    is_below_min_range: typing.Optional[bool] = None
    interpretation: typing.Optional[str] = None
    loinc: typing.Optional[str] = None
    loinc_slug: typing.Optional[str] = None
    provider_id: typing.Optional[str] = None
    source_markers: typing.Optional[typing.List[ParentBiomarkerData]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingInsulinInjectionSampleType(str, enum.Enum):
    """
    Insulin type: rapid vs long acting
    """

    RAPID_ACTING = "rapid_acting"
    LONG_ACTING = "long_acting"

    def visit(
        self, rapid_acting: typing.Callable[[], T_Result], long_acting: typing.Callable[[], T_Result]
    ) -> T_Result:
        if self is ClientFacingInsulinInjectionSampleType.RAPID_ACTING:
            return rapid_acting()
        if self is ClientFacingInsulinInjectionSampleType.LONG_ACTING:
            return long_acting()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class BodyColumnExprBody(str, enum.Enum):
    MEASURED_AT = "measured_at"
    WEIGHT_KILOGRAM = "weight_kilogram"
    FAT_MASS_PERCENTAGE = "fat_mass_percentage"
    WATER_PERCENTAGE = "water_percentage"
    MUSCLE_MASS_PERCENTAGE = "muscle_mass_percentage"
    VISCERAL_FAT_INDEX = "visceral_fat_index"
    BONE_MASS_PERCENTAGE = "bone_mass_percentage"
    BODY_MASS_INDEX = "body_mass_index"
    LEAN_BODY_MASS_KILOGRAM = "lean_body_mass_kilogram"
    WAIST_CIRCUMFERENCE_CENTIMETER = "waist_circumference_centimeter"
    SOURCE_TYPE = "source_type"
    SOURCE_PROVIDER = "source_provider"
    SOURCE_APP_ID = "source_app_id"
    TIME_ZONE = "time_zone"

    def visit(
        self,
        measured_at: typing.Callable[[], T_Result],
        weight_kilogram: typing.Callable[[], T_Result],
        fat_mass_percentage: typing.Callable[[], T_Result],
        water_percentage: typing.Callable[[], T_Result],
        muscle_mass_percentage: typing.Callable[[], T_Result],
        visceral_fat_index: typing.Callable[[], T_Result],
        bone_mass_percentage: typing.Callable[[], T_Result],
        body_mass_index: typing.Callable[[], T_Result],
        lean_body_mass_kilogram: typing.Callable[[], T_Result],
        waist_circumference_centimeter: typing.Callable[[], T_Result],
        source_type: typing.Callable[[], T_Result],
        source_provider: typing.Callable[[], T_Result],
        source_app_id: typing.Callable[[], T_Result],
        time_zone: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is BodyColumnExprBody.MEASURED_AT:
            return measured_at()
        if self is BodyColumnExprBody.WEIGHT_KILOGRAM:
            return weight_kilogram()
        if self is BodyColumnExprBody.FAT_MASS_PERCENTAGE:
            return fat_mass_percentage()
        if self is BodyColumnExprBody.WATER_PERCENTAGE:
            return water_percentage()
        if self is BodyColumnExprBody.MUSCLE_MASS_PERCENTAGE:
            return muscle_mass_percentage()
        if self is BodyColumnExprBody.VISCERAL_FAT_INDEX:
            return visceral_fat_index()
        if self is BodyColumnExprBody.BONE_MASS_PERCENTAGE:
            return bone_mass_percentage()
        if self is BodyColumnExprBody.BODY_MASS_INDEX:
            return body_mass_index()
        if self is BodyColumnExprBody.LEAN_BODY_MASS_KILOGRAM:
            return lean_body_mass_kilogram()
        if self is BodyColumnExprBody.WAIST_CIRCUMFERENCE_CENTIMETER:
            return waist_circumference_centimeter()
        if self is BodyColumnExprBody.SOURCE_TYPE:
            return source_type()
        if self is BodyColumnExprBody.SOURCE_PROVIDER:
            return source_provider()
        if self is BodyColumnExprBody.SOURCE_APP_ID:
            return source_app_id()
        if self is BodyColumnExprBody.TIME_ZONE:
            return time_zone()



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ClientFacingShipment(UniversalBaseModel):
    """
    Schema for a Shipment in the client facing API.

    To be used as part of a ClientFacingTestkitOrder.
    """

    id: str = pydantic.Field()
    """
    The Vital Shipment ID
    """

    outbound_tracking_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Tracking number for delivery to customer
    """

    outbound_tracking_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Tracking url for delivery to customer
    """

    inbound_tracking_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Tracking number for delivery to lab
    """

    inbound_tracking_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Tracking url for delivery to lab
    """

    outbound_courier: typing.Optional[str] = pydantic.Field(default=None)
    """
    Courier used for delivery to customer
    """

    inbound_courier: typing.Optional[str] = pydantic.Field(default=None)
    """
    Courier used for delivery to lab
    """

    notes: typing.Optional[str] = pydantic.Field(default=None)
    """
    Notes associated to the Vital shipment
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingWorkoutDurationSampleIntensity(str, enum.Enum):
    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"

    def visit(
        self,
        low: typing.Callable[[], T_Result],
        medium: typing.Callable[[], T_Result],
        high: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingWorkoutDurationSampleIntensity.LOW:
            return low()
        if self is ClientFacingWorkoutDurationSampleIntensity.MEDIUM:
            return medium()
        if self is ClientFacingWorkoutDurationSampleIntensity.HIGH:
            return high()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingForcedVitalCapacityChangedEventType(str, enum.Enum):
    DAILY_DATA_FORCED_VITAL_CAPACITY_CREATED = "daily.data.forced_vital_capacity.created"
    DAILY_DATA_FORCED_VITAL_CAPACITY_UPDATED = "daily.data.forced_vital_capacity.updated"

    def visit(
        self,
        daily_data_forced_vital_capacity_created: typing.Callable[[], T_Result],
        daily_data_forced_vital_capacity_updated: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingForcedVitalCapacityChangedEventType.DAILY_DATA_FORCED_VITAL_CAPACITY_CREATED:
            return daily_data_forced_vital_capacity_created()
        if self is ClientFacingForcedVitalCapacityChangedEventType.DAILY_DATA_FORCED_VITAL_CAPACITY_UPDATED:
            return daily_data_forced_vital_capacity_updated()



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .client_facing_patient_details_compatible import ClientFacingPatientDetailsCompatible
from .patient_address_compatible import PatientAddressCompatible
from .client_facing_lab_test import ClientFacingLabTest
from .client_facing_order_details import ClientFacingOrderDetails
import datetime as dt
from .client_facing_order_event import ClientFacingOrderEvent
from .order_top_level_status import OrderTopLevelStatus
from .client_facing_physician import ClientFacingPhysician
from .shipping_address import ShippingAddress
from .billing import Billing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ClientFacingOrder(UniversalBaseModel):
    user_id: str = pydantic.Field()
    """
    User id returned by vital create user request. This id should be stored in your database against the user and used for all interactions with the vital api.
    """

    id: str = pydantic.Field()
    """
    The Vital Order ID
    """

    team_id: str = pydantic.Field()
    """
    Your team id.
    """

    patient_details: typing.Optional[ClientFacingPatientDetailsCompatible] = pydantic.Field(default=None)
    """
    Patient Details
    """

    patient_address: typing.Optional[PatientAddressCompatible] = pydantic.Field(default=None)
    """
    Patient Address
    """

    lab_test: ClientFacingLabTest = pydantic.Field()
    """
    The Vital Test associated with the order
    """

    details: ClientFacingOrderDetails
    sample_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Sample ID
    """

    notes: typing.Optional[str] = pydantic.Field(default=None)
    """
    Notes associated with the order
    """

    created_at: dt.datetime = pydantic.Field()
    """
    When your order was created
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    When your order was last updated.
    """

    events: typing.List[ClientFacingOrderEvent]
    status: typing.Optional[OrderTopLevelStatus] = None
    physician: typing.Optional[ClientFacingPhysician] = None
    health_insurance_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Vital ID of the health insurance.
    """

    requisition_form_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    DEPRECATED. Requistion form url.
    """

    priority: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Defines whether order is priority or not. For some labs, this refers to a STAT order.
    """

    shipping_details: typing.Optional[ShippingAddress] = pydantic.Field(default=None)
    """
    Shipping Details. For unregistered testkit orders.
    """

    activate_by: typing.Optional[str] = pydantic.Field(default=None)
    """
    Schedule an Order to be processed in a future date.
    """

    passthrough: typing.Optional[str] = None
    billing_type: typing.Optional[Billing] = None
    icd_codes: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    LabTestsGetOrdersRequestOrderDirection,
    LabTestsGetOrdersRequestOrderKey,
    LabTestsGetPaginatedRequestOrderDirection,
    LabTestsGetPaginatedRequestOrderKey,
    LabTestsGetRequestOrderDirection,
    LabTestsGetRequestOrderKey,
)

__all__ = [
    "LabTestsGetOrdersRequestOrderDirection",
    "LabTestsGetOrdersRequestOrderKey",
    "LabTestsGetPaginatedRequestOrderDirection",
    "LabTestsGetPaginatedRequestOrderKey",
    "LabTestsGetRequestOrderDirection",
    "LabTestsGetRequestOrderKey",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingBodyMassIndexChangedEventType(str, enum.Enum):
    DAILY_DATA_BODY_MASS_INDEX_CREATED = "daily.data.body_mass_index.created"
    DAILY_DATA_BODY_MASS_INDEX_UPDATED = "daily.data.body_mass_index.updated"

    def visit(
        self,
        daily_data_body_mass_index_created: typing.Callable[[], T_Result],
        daily_data_body_mass_index_updated: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingBodyMassIndexChangedEventType.DAILY_DATA_BODY_MASS_INDEX_CREATED:
            return daily_data_body_mass_index_created()
        if self is ClientFacingBodyMassIndexChangedEventType.DAILY_DATA_BODY_MASS_INDEX_UPDATED:
            return daily_data_body_mass_index_updated()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AllowedRadius(str, enum.Enum):
    TEN = "10"
    TWENTY = "20"
    TWENTY_FIVE = "25"
    FIFTY = "50"

    def visit(
        self,
        ten: typing.Callable[[], T_Result],
        twenty: typing.Callable[[], T_Result],
        twenty_five: typing.Callable[[], T_Result],
        fifty: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is AllowedRadius.TEN:
            return ten()
        if self is AllowedRadius.TWENTY:
            return twenty()
        if self is AllowedRadius.TWENTY_FIVE:
            return twenty_five()
        if self is AllowedRadius.FIFTY:
            return fifty()



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class ClientFacingCriticalResult(UniversalBaseModel):
    order_id: str = pydantic.Field()
    """
    The Vital Order ID
    """

    sample_id: str = pydantic.Field()
    """
    Sample ID
    """

    status: str = pydantic.Field()
    """
    The current results status, can be either (final, partial).
    """

    interpretation: str = pydantic.Field()
    """
    The results interpretation, can be either (normal, abnormal, critical).
    """

    team_id: str = pydantic.Field()
    """
    Your team id.
    """

    user_id: str
    created_at: dt.datetime = pydantic.Field()
    """
    When the result was first fetched from a laboratory.
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    When the result was last fetched from a laboratory.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class TraceElements(str, enum.Enum):
    CHROMIUM = "chromium"
    COPPER = "copper"
    IODINE = "iodine"
    MANGANESE = "manganese"
    MOLYBDENUM = "molybdenum"
    SELENIUM = "selenium"

    def visit(
        self,
        chromium: typing.Callable[[], T_Result],
        copper: typing.Callable[[], T_Result],
        iodine: typing.Callable[[], T_Result],
        manganese: typing.Callable[[], T_Result],
        molybdenum: typing.Callable[[], T_Result],
        selenium: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is TraceElements.CHROMIUM:
            return chromium()
        if self is TraceElements.COPPER:
            return copper()
        if self is TraceElements.IODINE:
            return iodine()
        if self is TraceElements.MANGANESE:
            return manganese()
        if self is TraceElements.MOLYBDENUM:
            return molybdenum()
        if self is TraceElements.SELENIUM:
            return selenium()



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .grouped_body_mass_index import GroupedBodyMassIndex
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class GroupedBodyMassIndexResponse(UniversalBaseModel):
    groups: typing.Dict[str, typing.List[GroupedBodyMassIndex]] = pydantic.Field()
    """
    For each matching provider or lab, a list of grouped timeseries values.
    """

    next: typing.Optional[str] = pydantic.Field(default=None)
    """
    The cursor for fetching the next page, or `null` if there is no more data.
    """

    next_cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    The cursor for fetching the next page, or `null` if there is no more data.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingResource(str, enum.Enum):
    PROFILE = "profile"
    ACTIVITY = "activity"
    SLEEP = "sleep"
    BODY = "body"
    WORKOUTS = "workouts"
    WORKOUT_STREAM = "workout_stream"
    CONNECTION = "connection"
    ORDER = "order"
    RESULT = "result"
    APPOINTMENT = "appointment"
    GLUCOSE = "glucose"
    HEARTRATE = "heartrate"
    HRV = "hrv"
    IGE = "ige"
    IGG = "igg"
    BLOOD_OXYGEN = "blood_oxygen"
    BLOOD_PRESSURE = "blood_pressure"
    CHOLESTEROL = "cholesterol"
    DEVICE = "device"
    WEIGHT = "weight"
    FAT = "fat"
    BODY_TEMPERATURE = "body_temperature"
    BODY_TEMPERATURE_DELTA = "body_temperature_delta"
    MEAL = "meal"
    WATER = "water"
    CAFFEINE = "caffeine"
    MINDFULNESS_MINUTES = "mindfulness_minutes"
    STEPS = "steps"
    CALORIES_ACTIVE = "calories_active"
    DISTANCE = "distance"
    FLOORS_CLIMBED = "floors_climbed"
    RESPIRATORY_RATE = "respiratory_rate"
    VO_2_MAX = "vo2_max"
    CALORIES_BASAL = "calories_basal"
    STRESS_LEVEL = "stress_level"
    MENSTRUAL_CYCLE = "menstrual_cycle"
    SLEEP_CYCLE = "sleep_cycle"
    ELECTROCARDIOGRAM = "electrocardiogram"
    ELECTROCARDIOGRAM_VOLTAGE = "electrocardiogram_voltage"
    AFIB_BURDEN = "afib_burden"
    HEART_RATE_ALERT = "heart_rate_alert"
    STAND_HOUR = "stand_hour"
    STAND_DURATION = "stand_duration"
    SLEEP_APNEA_ALERT = "sleep_apnea_alert"
    SLEEP_BREATHING_DISTURBANCE = "sleep_breathing_disturbance"
    WHEELCHAIR_PUSH = "wheelchair_push"
    FORCED_EXPIRATORY_VOLUME_1 = "forced_expiratory_volume_1"
    FORCED_VITAL_CAPACITY = "forced_vital_capacity"
    PEAK_EXPIRATORY_FLOW_RATE = "peak_expiratory_flow_rate"
    INHALER_USAGE = "inhaler_usage"
    FALL = "fall"
    UV_EXPOSURE = "uv_exposure"
    DAYLIGHT_EXPOSURE = "daylight_exposure"
    HANDWASHING = "handwashing"
    BASAL_BODY_TEMPERATURE = "basal_body_temperature"
    BODY_MASS_INDEX = "body_mass_index"
    LEAN_BODY_MASS = "lean_body_mass"
    WAIST_CIRCUMFERENCE = "waist_circumference"
    WORKOUT_DURATION = "workout_duration"
    INSULIN_INJECTION = "insulin_injection"
    CARBOHYDRATES = "carbohydrates"
    NOTE = "note"
    SLEEP_STREAM = "sleep_stream"
    HYPNOGRAM = "hypnogram"

    def visit(
        self,
        profile: typing.Callable[[], T_Result],
        activity: typing.Callable[[], T_Result],
        sleep: typing.Callable[[], T_Result],
        body: typing.Callable[[], T_Result],
        workouts: typing.Callable[[], T_Result],
        workout_stream: typing.Callable[[], T_Result],
        connection: typing.Callable[[], T_Result],
        order: typing.Callable[[], T_Result],
        result: typing.Callable[[], T_Result],
        appointment: typing.Callable[[], T_Result],
        glucose: typing.Callable[[], T_Result],
        heartrate: typing.Callable[[], T_Result],
        hrv: typing.Callable[[], T_Result],
        ige: typing.Callable[[], T_Result],
        igg: typing.Callable[[], T_Result],
        blood_oxygen: typing.Callable[[], T_Result],
        blood_pressure: typing.Callable[[], T_Result],
        cholesterol: typing.Callable[[], T_Result],
        device: typing.Callable[[], T_Result],
        weight: typing.Callable[[], T_Result],
        fat: typing.Callable[[], T_Result],
        body_temperature: typing.Callable[[], T_Result],
        body_temperature_delta: typing.Callable[[], T_Result],
        meal: typing.Callable[[], T_Result],
        water: typing.Callable[[], T_Result],
        caffeine: typing.Callable[[], T_Result],
        mindfulness_minutes: typing.Callable[[], T_Result],
        steps: typing.Callable[[], T_Result],
        calories_active: typing.Callable[[], T_Result],
        distance: typing.Callable[[], T_Result],
        floors_climbed: typing.Callable[[], T_Result],
        respiratory_rate: typing.Callable[[], T_Result],
        vo_2_max: typing.Callable[[], T_Result],
        calories_basal: typing.Callable[[], T_Result],
        stress_level: typing.Callable[[], T_Result],
        menstrual_cycle: typing.Callable[[], T_Result],
        sleep_cycle: typing.Callable[[], T_Result],
        electrocardiogram: typing.Callable[[], T_Result],
        electrocardiogram_voltage: typing.Callable[[], T_Result],
        afib_burden: typing.Callable[[], T_Result],
        heart_rate_alert: typing.Callable[[], T_Result],
        stand_hour: typing.Callable[[], T_Result],
        stand_duration: typing.Callable[[], T_Result],
        sleep_apnea_alert: typing.Callable[[], T_Result],
        sleep_breathing_disturbance: typing.Callable[[], T_Result],
        wheelchair_push: typing.Callable[[], T_Result],
        forced_expiratory_volume_1: typing.Callable[[], T_Result],
        forced_vital_capacity: typing.Callable[[], T_Result],
        peak_expiratory_flow_rate: typing.Callable[[], T_Result],
        inhaler_usage: typing.Callable[[], T_Result],
        fall: typing.Callable[[], T_Result],
        uv_exposure: typing.Callable[[], T_Result],
        daylight_exposure: typing.Callable[[], T_Result],
        handwashing: typing.Callable[[], T_Result],
        basal_body_temperature: typing.Callable[[], T_Result],
        body_mass_index: typing.Callable[[], T_Result],
        lean_body_mass: typing.Callable[[], T_Result],
        waist_circumference: typing.Callable[[], T_Result],
        workout_duration: typing.Callable[[], T_Result],
        insulin_injection: typing.Callable[[], T_Result],
        carbohydrates: typing.Callable[[], T_Result],
        note: typing.Callable[[], T_Result],
        sleep_stream: typing.Callable[[], T_Result],
        hypnogram: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingResource.PROFILE:
            return profile()
        if self is ClientFacingResource.ACTIVITY:
            return activity()
        if self is ClientFacingResource.SLEEP:
            return sleep()
        if self is ClientFacingResource.BODY:
            return body()
        if self is ClientFacingResource.WORKOUTS:
            return workouts()
        if self is ClientFacingResource.WORKOUT_STREAM:
            return workout_stream()
        if self is ClientFacingResource.CONNECTION:
            return connection()
        if self is ClientFacingResource.ORDER:
            return order()
        if self is ClientFacingResource.RESULT:
            return result()
        if self is ClientFacingResource.APPOINTMENT:
            return appointment()
        if self is ClientFacingResource.GLUCOSE:
            return glucose()
        if self is ClientFacingResource.HEARTRATE:
            return heartrate()
        if self is ClientFacingResource.HRV:
            return hrv()
        if self is ClientFacingResource.IGE:
            return ige()
        if self is ClientFacingResource.IGG:
            return igg()
        if self is ClientFacingResource.BLOOD_OXYGEN:
            return blood_oxygen()
        if self is ClientFacingResource.BLOOD_PRESSURE:
            return blood_pressure()
        if self is ClientFacingResource.CHOLESTEROL:
            return cholesterol()
        if self is ClientFacingResource.DEVICE:
            return device()
        if self is ClientFacingResource.WEIGHT:
            return weight()
        if self is ClientFacingResource.FAT:
            return fat()
        if self is ClientFacingResource.BODY_TEMPERATURE:
            return body_temperature()
        if self is ClientFacingResource.BODY_TEMPERATURE_DELTA:
            return body_temperature_delta()
        if self is ClientFacingResource.MEAL:
            return meal()
        if self is ClientFacingResource.WATER:
            return water()
        if self is ClientFacingResource.CAFFEINE:
            return caffeine()
        if self is ClientFacingResource.MINDFULNESS_MINUTES:
            return mindfulness_minutes()
        if self is ClientFacingResource.STEPS:
            return steps()
        if self is ClientFacingResource.CALORIES_ACTIVE:
            return calories_active()
        if self is ClientFacingResource.DISTANCE:
            return distance()
        if self is ClientFacingResource.FLOORS_CLIMBED:
            return floors_climbed()
        if self is ClientFacingResource.RESPIRATORY_RATE:
            return respiratory_rate()
        if self is ClientFacingResource.VO_2_MAX:
            return vo_2_max()
        if self is ClientFacingResource.CALORIES_BASAL:
            return calories_basal()
        if self is ClientFacingResource.STRESS_LEVEL:
            return stress_level()
        if self is ClientFacingResource.MENSTRUAL_CYCLE:
            return menstrual_cycle()
        if self is ClientFacingResource.SLEEP_CYCLE:
            return sleep_cycle()
        if self is ClientFacingResource.ELECTROCARDIOGRAM:
            return electrocardiogram()
        if self is ClientFacingResource.ELECTROCARDIOGRAM_VOLTAGE:
            return electrocardiogram_voltage()
        if self is ClientFacingResource.AFIB_BURDEN:
            return afib_burden()
        if self is ClientFacingResource.HEART_RATE_ALERT:
            return heart_rate_alert()
        if self is ClientFacingResource.STAND_HOUR:
            return stand_hour()
        if self is ClientFacingResource.STAND_DURATION:
            return stand_duration()
        if self is ClientFacingResource.SLEEP_APNEA_ALERT:
            return sleep_apnea_alert()
        if self is ClientFacingResource.SLEEP_BREATHING_DISTURBANCE:
            return sleep_breathing_disturbance()
        if self is ClientFacingResource.WHEELCHAIR_PUSH:
            return wheelchair_push()
        if self is ClientFacingResource.FORCED_EXPIRATORY_VOLUME_1:
            return forced_expiratory_volume_1()
        if self is ClientFacingResource.FORCED_VITAL_CAPACITY:
            return forced_vital_capacity()
        if self is ClientFacingResource.PEAK_EXPIRATORY_FLOW_RATE:
            return peak_expiratory_flow_rate()
        if self is ClientFacingResource.INHALER_USAGE:
            return inhaler_usage()
        if self is ClientFacingResource.FALL:
            return fall()
        if self is ClientFacingResource.UV_EXPOSURE:
            return uv_exposure()
        if self is ClientFacingResource.DAYLIGHT_EXPOSURE:
            return daylight_exposure()
        if self is ClientFacingResource.HANDWASHING:
            return handwashing()
        if self is ClientFacingResource.BASAL_BODY_TEMPERATURE:
            return basal_body_temperature()
        if self is ClientFacingResource.BODY_MASS_INDEX:
            return body_mass_index()
        if self is ClientFacingResource.LEAN_BODY_MASS:
            return lean_body_mass()
        if self is ClientFacingResource.WAIST_CIRCUMFERENCE:
            return waist_circumference()
        if self is ClientFacingResource.WORKOUT_DURATION:
            return workout_duration()
        if self is ClientFacingResource.INSULIN_INJECTION:
            return insulin_injection()
        if self is ClientFacingResource.CARBOHYDRATES:
            return carbohydrates()
        if self is ClientFacingResource.NOTE:
            return note()
        if self is ClientFacingResource.SLEEP_STREAM:
            return sleep_stream()
        if self is ClientFacingResource.HYPNOGRAM:
            return hypnogram()



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class FallbackBirthDate(UniversalBaseModel):
    value: str = pydantic.Field()
    """
    Fallback date of birth of the user, in YYYY-mm-dd format. Used for calculating max heartrate for providers that don not provide users' age.
    """

    source_slug: str = pydantic.Field()
    """
    Slug for designated source
    """

    updated_at: dt.datetime

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingWorkoutStreamChangedEventType(str, enum.Enum):
    DAILY_DATA_WORKOUT_STREAM_CREATED = "daily.data.workout_stream.created"
    DAILY_DATA_WORKOUT_STREAM_UPDATED = "daily.data.workout_stream.updated"

    def visit(
        self,
        daily_data_workout_stream_created: typing.Callable[[], T_Result],
        daily_data_workout_stream_updated: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingWorkoutStreamChangedEventType.DAILY_DATA_WORKOUT_STREAM_CREATED:
            return daily_data_workout_stream_created()
        if self is ClientFacingWorkoutStreamChangedEventType.DAILY_DATA_WORKOUT_STREAM_UPDATED:
            return daily_data_workout_stream_updated()



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..types.o_auth_providers import OAuthProviders
from ..types.connection_recipe import ConnectionRecipe
from ..core.request_options import RequestOptions
from ..types.bulk_import_connections_response import BulkImportConnectionsResponse
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.bulk_export_connections_response import BulkExportConnectionsResponse
from ..types.providers import Providers
from ..types.link_token_exchange_response import LinkTokenExchangeResponse
import datetime as dt
from ..types.vital_token_created_response import VitalTokenCreatedResponse
from ..core.datetime_utils import serialize_datetime
from ..types.auth_type import AuthType
from ..types.region import Region
from ..types.source import Source
from ..core.jsonable_encoder import jsonable_encoder
from ..types.password_providers import PasswordProviders
from ..types.provider_link_response import ProviderLinkResponse
from ..types.source_link import SourceLink
from ..types.manual_providers import ManualProviders
from ..types.demo_providers import DemoProviders
from ..types.demo_connection_status import DemoConnectionStatus
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class LinkClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def bulk_import(
        self,
        *,
        provider: OAuthProviders,
        connections: typing.Sequence[ConnectionRecipe],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkImportConnectionsResponse:
        """
        Parameters
        ----------
        provider : OAuthProviders

        connections : typing.Sequence[ConnectionRecipe]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkImportConnectionsResponse
            Successful Response

        Examples
        --------
        from vital import ConnectionRecipe, OAuthProviders, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.bulk_import(
            provider=OAuthProviders.OURA,
            connections=[
                ConnectionRecipe(
                    user_id="user_id",
                    access_token="access_token",
                    refresh_token="refresh_token",
                    provider_id="provider_id",
                    expires_at=1,
                )
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/link/bulk_import",
            method="POST",
            json={
                "provider": provider,
                "connections": connections,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BulkImportConnectionsResponse,
                    parse_obj_as(
                        type_=BulkImportConnectionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def bulk_trigger_historical_pull(
        self,
        *,
        user_ids: typing.Sequence[str],
        provider: OAuthProviders,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        user_ids : typing.Sequence[str]

        provider : OAuthProviders

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import OAuthProviders, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.bulk_trigger_historical_pull(
            user_ids=["user_ids"],
            provider=OAuthProviders.OURA,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/link/bulk_trigger_historical_pull",
            method="POST",
            json={
                "user_ids": user_ids,
                "provider": provider,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def bulk_export(
        self,
        *,
        provider: OAuthProviders,
        user_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        next_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkExportConnectionsResponse:
        """
        Parameters
        ----------
        provider : OAuthProviders

        user_ids : typing.Optional[typing.Sequence[str]]

        next_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkExportConnectionsResponse
            Successful Response

        Examples
        --------
        from vital import OAuthProviders, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.bulk_export(
            provider=OAuthProviders.OURA,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/link/bulk_export",
            method="POST",
            json={
                "user_ids": user_ids,
                "provider": provider,
                "next_token": next_token,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BulkExportConnectionsResponse,
                    parse_obj_as(
                        type_=BulkExportConnectionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def bulk_pause(
        self,
        *,
        user_ids: typing.Sequence[str],
        provider: OAuthProviders,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        user_ids : typing.Sequence[str]

        provider : OAuthProviders

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import OAuthProviders, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.bulk_pause(
            user_ids=["user_ids"],
            provider=OAuthProviders.OURA,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/link/bulk_pause",
            method="POST",
            json={
                "user_ids": user_ids,
                "provider": provider,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def token(
        self,
        *,
        user_id: str,
        provider: typing.Optional[Providers] = OMIT,
        redirect_url: typing.Optional[str] = OMIT,
        filter_on_providers: typing.Optional[typing.Sequence[Providers]] = OMIT,
        on_error: typing.Optional[typing.Literal["redirect"]] = OMIT,
        on_close: typing.Optional[typing.Literal["redirect"]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LinkTokenExchangeResponse:
        """
        Endpoint to generate a user link token, to be used throughout the vital
        link process. The vital link token is a one time use token, that
        expires after 10 minutes. If you would like vital-link widget to launch
        with a specific provider, pass in a provider in the body. If you would
        like to redirect to a custom url after successful or error connection,
        pass in your own custom redirect_url parameter.

        Parameters
        ----------
        user_id : str
            User id returned by vital create user request. This id should be stored in your database against the user and used for all interactions with the vital api.

        provider : typing.Optional[Providers]

        redirect_url : typing.Optional[str]

        filter_on_providers : typing.Optional[typing.Sequence[Providers]]

        on_error : typing.Optional[typing.Literal["redirect"]]

        on_close : typing.Optional[typing.Literal["redirect"]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LinkTokenExchangeResponse
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.token(
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/link/token",
            method="POST",
            json={
                "user_id": user_id,
                "provider": provider,
                "redirect_url": redirect_url,
                "filter_on_providers": filter_on_providers,
                "on_error": on_error,
                "on_close": on_close,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    LinkTokenExchangeResponse,
                    parse_obj_as(
                        type_=LinkTokenExchangeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def is_token_valid(
        self, *, token: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        token : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.is_token_valid(
            token="token",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/link/token/isValid",
            method="POST",
            json={
                "token": token,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def code_create(
        self,
        *,
        user_id: str,
        expires_at: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VitalTokenCreatedResponse:
        """
        Generate a token to invite a user of Vital mobile app to your team

        Parameters
        ----------
        user_id : str

        expires_at : typing.Optional[dt.datetime]
            When the link code should expire. Defaults to server time plus 1 hour.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VitalTokenCreatedResponse
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.code_create(
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/link/code/create",
            method="POST",
            params={
                "user_id": user_id,
                "expires_at": serialize_datetime(expires_at) if expires_at is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    VitalTokenCreatedResponse,
                    parse_obj_as(
                        type_=VitalTokenCreatedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def start_connect(
        self, *, link_token: str, provider: Providers, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        REQUEST_SOURCE: VITAL-LINK
        Start link token process

        Parameters
        ----------
        link_token : str

        provider : Providers

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        from vital import Providers, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.start_connect(
            link_token="link_token",
            provider=Providers.OURA,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/link/start",
            method="POST",
            json={
                "link_token": link_token,
                "provider": provider,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def token_state(
        self, *, vital_link_token: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        REQUEST_SOURCE: VITAL-LINK
        Check link token state - can be hit continuously used as heartbeat

        Parameters
        ----------
        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.token_state()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/link/state",
            method="GET",
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def email_auth(
        self,
        *,
        email: str,
        provider: Providers,
        auth_type: AuthType,
        vital_link_token: typing.Optional[str] = None,
        region: typing.Optional[Region] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Deprecated. Use `POST /v2/link/provider/email/{provider}` instead.

        Parameters
        ----------
        email : str

        provider : Providers

        auth_type : AuthType

        vital_link_token : typing.Optional[str]

        region : typing.Optional[Region]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import AuthType, Providers, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.email_auth(
            email="email",
            provider=Providers.OURA,
            auth_type=AuthType.PASSWORD,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/link/auth/email",
            method="POST",
            json={
                "email": email,
                "provider": provider,
                "auth_type": auth_type,
                "region": region,
            },
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def password_auth(
        self,
        *,
        username: str,
        password: str,
        provider: Providers,
        auth_type: AuthType,
        vital_link_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Deprecated. Use `POST /v2/link/provider/password/{provider}` instead.

        Parameters
        ----------
        username : str

        password : str

        provider : Providers

        auth_type : AuthType

        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import AuthType, Providers, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.password_auth(
            username="username",
            password="password",
            provider=Providers.OURA,
            auth_type=AuthType.PASSWORD,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/link/auth",
            method="POST",
            json={
                "username": username,
                "password": password,
                "provider": provider,
                "auth_type": auth_type,
            },
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def generate_oauth_link(
        self,
        oauth_provider: OAuthProviders,
        *,
        vital_link_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Source:
        """
        This endpoint generates an OAuth link for oauth provider

        Parameters
        ----------
        oauth_provider : OAuthProviders

        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Source
            Successful Response

        Examples
        --------
        from vital import OAuthProviders, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.generate_oauth_link(
            oauth_provider=OAuthProviders.OURA,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/link/provider/oauth/{jsonable_encoder(oauth_provider)}",
            method="GET",
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Source,
                    parse_obj_as(
                        type_=Source,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def connect_password_provider(
        self,
        provider: PasswordProviders,
        *,
        username: str,
        password: str,
        vital_link_token: typing.Optional[str] = None,
        region: typing.Optional[Region] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProviderLinkResponse:
        """
        This connects auth providers that are password based.

        Parameters
        ----------
        provider : PasswordProviders

        username : str
            Username for provider

        password : str
            Password for provider

        vital_link_token : typing.Optional[str]

        region : typing.Optional[Region]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProviderLinkResponse
            Successful Response

        Examples
        --------
        from vital import PasswordProviders, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.connect_password_provider(
            provider=PasswordProviders.WHOOP,
            username="username",
            password="password",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/link/provider/password/{jsonable_encoder(provider)}",
            method="POST",
            json={
                "username": username,
                "password": password,
                "region": region,
            },
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProviderLinkResponse,
                    parse_obj_as(
                        type_=ProviderLinkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def complete_password_provider_mfa(
        self,
        provider: PasswordProviders,
        *,
        mfa_code: str,
        vital_link_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProviderLinkResponse:
        """
        This connects auth providers that are password based.

        Parameters
        ----------
        provider : PasswordProviders

        mfa_code : str

        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProviderLinkResponse
            Successful Response

        Examples
        --------
        from vital import PasswordProviders, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.complete_password_provider_mfa(
            provider=PasswordProviders.WHOOP,
            mfa_code="mfa_code",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/link/provider/password/{jsonable_encoder(provider)}/complete_mfa",
            method="POST",
            json={
                "mfa_code": mfa_code,
            },
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProviderLinkResponse,
                    parse_obj_as(
                        type_=ProviderLinkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def connect_email_auth_provider(
        self,
        *,
        email: str,
        vital_link_token: typing.Optional[str] = None,
        email_provider_auth_link_provider: typing.Optional[Providers] = OMIT,
        region: typing.Optional[Region] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        This connects auth providers that are email based.

        Parameters
        ----------
        email : str

        vital_link_token : typing.Optional[str]

        email_provider_auth_link_provider : typing.Optional[Providers]

        region : typing.Optional[Region]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.connect_email_auth_provider(
            email="email",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/link/provider/email/freestyle_libre",
            method="POST",
            json={
                "email": email,
                "provider": email_provider_auth_link_provider,
                "region": region,
            },
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_all_providers(
        self, *, vital_link_token: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[SourceLink]:
        """
        GET List of all available providers given the generated link token.

        Parameters
        ----------
        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SourceLink]
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.get_all_providers()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/link/providers",
            method="GET",
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[SourceLink],
                    parse_obj_as(
                        type_=typing.List[SourceLink],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def connect_manual_provider(
        self,
        provider: ManualProviders,
        *,
        user_id: str,
        provider_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, bool]:
        """
        Parameters
        ----------
        provider : ManualProviders

        user_id : str

        provider_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, bool]
            Successful Response

        Examples
        --------
        from vital import ManualProviders, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.connect_manual_provider(
            provider=ManualProviders.BEURER_BLE,
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/link/provider/manual/{jsonable_encoder(provider)}",
            method="POST",
            json={
                "user_id": user_id,
                "provider_id": provider_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, bool],
                    parse_obj_as(
                        type_=typing.Dict[str, bool],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def connect_demo_provider(
        self, *, user_id: str, provider: DemoProviders, request_options: typing.Optional[RequestOptions] = None
    ) -> DemoConnectionStatus:
        """
        POST Connect the given Vital user to a demo provider.

        Parameters
        ----------
        user_id : str
            Vital user ID

        provider : DemoProviders
            Demo provider. For more information, please check out our docs (https://docs.tryvital.io/wearables/providers/test_data)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DemoConnectionStatus
            Successful Response

        Examples
        --------
        from vital import DemoProviders, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.link.connect_demo_provider(
            user_id="user_id",
            provider=DemoProviders.APPLE_HEALTH_KIT,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/link/connect/demo",
            method="POST",
            json={
                "user_id": user_id,
                "provider": provider,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    DemoConnectionStatus,
                    parse_obj_as(
                        type_=DemoConnectionStatus,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncLinkClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def bulk_import(
        self,
        *,
        provider: OAuthProviders,
        connections: typing.Sequence[ConnectionRecipe],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkImportConnectionsResponse:
        """
        Parameters
        ----------
        provider : OAuthProviders

        connections : typing.Sequence[ConnectionRecipe]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkImportConnectionsResponse
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, ConnectionRecipe, OAuthProviders

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.bulk_import(
                provider=OAuthProviders.OURA,
                connections=[
                    ConnectionRecipe(
                        user_id="user_id",
                        access_token="access_token",
                        refresh_token="refresh_token",
                        provider_id="provider_id",
                        expires_at=1,
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/link/bulk_import",
            method="POST",
            json={
                "provider": provider,
                "connections": connections,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BulkImportConnectionsResponse,
                    parse_obj_as(
                        type_=BulkImportConnectionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def bulk_trigger_historical_pull(
        self,
        *,
        user_ids: typing.Sequence[str],
        provider: OAuthProviders,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        user_ids : typing.Sequence[str]

        provider : OAuthProviders

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, OAuthProviders

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.bulk_trigger_historical_pull(
                user_ids=["user_ids"],
                provider=OAuthProviders.OURA,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/link/bulk_trigger_historical_pull",
            method="POST",
            json={
                "user_ids": user_ids,
                "provider": provider,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def bulk_export(
        self,
        *,
        provider: OAuthProviders,
        user_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        next_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkExportConnectionsResponse:
        """
        Parameters
        ----------
        provider : OAuthProviders

        user_ids : typing.Optional[typing.Sequence[str]]

        next_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkExportConnectionsResponse
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, OAuthProviders

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.bulk_export(
                provider=OAuthProviders.OURA,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/link/bulk_export",
            method="POST",
            json={
                "user_ids": user_ids,
                "provider": provider,
                "next_token": next_token,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BulkExportConnectionsResponse,
                    parse_obj_as(
                        type_=BulkExportConnectionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def bulk_pause(
        self,
        *,
        user_ids: typing.Sequence[str],
        provider: OAuthProviders,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Parameters
        ----------
        user_ids : typing.Sequence[str]

        provider : OAuthProviders

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, OAuthProviders

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.bulk_pause(
                user_ids=["user_ids"],
                provider=OAuthProviders.OURA,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/link/bulk_pause",
            method="POST",
            json={
                "user_ids": user_ids,
                "provider": provider,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def token(
        self,
        *,
        user_id: str,
        provider: typing.Optional[Providers] = OMIT,
        redirect_url: typing.Optional[str] = OMIT,
        filter_on_providers: typing.Optional[typing.Sequence[Providers]] = OMIT,
        on_error: typing.Optional[typing.Literal["redirect"]] = OMIT,
        on_close: typing.Optional[typing.Literal["redirect"]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LinkTokenExchangeResponse:
        """
        Endpoint to generate a user link token, to be used throughout the vital
        link process. The vital link token is a one time use token, that
        expires after 10 minutes. If you would like vital-link widget to launch
        with a specific provider, pass in a provider in the body. If you would
        like to redirect to a custom url after successful or error connection,
        pass in your own custom redirect_url parameter.

        Parameters
        ----------
        user_id : str
            User id returned by vital create user request. This id should be stored in your database against the user and used for all interactions with the vital api.

        provider : typing.Optional[Providers]

        redirect_url : typing.Optional[str]

        filter_on_providers : typing.Optional[typing.Sequence[Providers]]

        on_error : typing.Optional[typing.Literal["redirect"]]

        on_close : typing.Optional[typing.Literal["redirect"]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LinkTokenExchangeResponse
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.token(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/link/token",
            method="POST",
            json={
                "user_id": user_id,
                "provider": provider,
                "redirect_url": redirect_url,
                "filter_on_providers": filter_on_providers,
                "on_error": on_error,
                "on_close": on_close,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    LinkTokenExchangeResponse,
                    parse_obj_as(
                        type_=LinkTokenExchangeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def is_token_valid(
        self, *, token: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        token : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.is_token_valid(
                token="token",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/link/token/isValid",
            method="POST",
            json={
                "token": token,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def code_create(
        self,
        *,
        user_id: str,
        expires_at: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VitalTokenCreatedResponse:
        """
        Generate a token to invite a user of Vital mobile app to your team

        Parameters
        ----------
        user_id : str

        expires_at : typing.Optional[dt.datetime]
            When the link code should expire. Defaults to server time plus 1 hour.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VitalTokenCreatedResponse
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.code_create(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/link/code/create",
            method="POST",
            params={
                "user_id": user_id,
                "expires_at": serialize_datetime(expires_at) if expires_at is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    VitalTokenCreatedResponse,
                    parse_obj_as(
                        type_=VitalTokenCreatedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def start_connect(
        self, *, link_token: str, provider: Providers, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        REQUEST_SOURCE: VITAL-LINK
        Start link token process

        Parameters
        ----------
        link_token : str

        provider : Providers

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, Providers

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.start_connect(
                link_token="link_token",
                provider=Providers.OURA,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/link/start",
            method="POST",
            json={
                "link_token": link_token,
                "provider": provider,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def token_state(
        self, *, vital_link_token: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        REQUEST_SOURCE: VITAL-LINK
        Check link token state - can be hit continuously used as heartbeat

        Parameters
        ----------
        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.token_state()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/link/state",
            method="GET",
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def email_auth(
        self,
        *,
        email: str,
        provider: Providers,
        auth_type: AuthType,
        vital_link_token: typing.Optional[str] = None,
        region: typing.Optional[Region] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Deprecated. Use `POST /v2/link/provider/email/{provider}` instead.

        Parameters
        ----------
        email : str

        provider : Providers

        auth_type : AuthType

        vital_link_token : typing.Optional[str]

        region : typing.Optional[Region]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, AuthType, Providers

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.email_auth(
                email="email",
                provider=Providers.OURA,
                auth_type=AuthType.PASSWORD,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/link/auth/email",
            method="POST",
            json={
                "email": email,
                "provider": provider,
                "auth_type": auth_type,
                "region": region,
            },
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def password_auth(
        self,
        *,
        username: str,
        password: str,
        provider: Providers,
        auth_type: AuthType,
        vital_link_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Deprecated. Use `POST /v2/link/provider/password/{provider}` instead.

        Parameters
        ----------
        username : str

        password : str

        provider : Providers

        auth_type : AuthType

        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, AuthType, Providers

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.password_auth(
                username="username",
                password="password",
                provider=Providers.OURA,
                auth_type=AuthType.PASSWORD,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/link/auth",
            method="POST",
            json={
                "username": username,
                "password": password,
                "provider": provider,
                "auth_type": auth_type,
            },
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def generate_oauth_link(
        self,
        oauth_provider: OAuthProviders,
        *,
        vital_link_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Source:
        """
        This endpoint generates an OAuth link for oauth provider

        Parameters
        ----------
        oauth_provider : OAuthProviders

        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Source
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, OAuthProviders

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.generate_oauth_link(
                oauth_provider=OAuthProviders.OURA,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/link/provider/oauth/{jsonable_encoder(oauth_provider)}",
            method="GET",
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Source,
                    parse_obj_as(
                        type_=Source,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def connect_password_provider(
        self,
        provider: PasswordProviders,
        *,
        username: str,
        password: str,
        vital_link_token: typing.Optional[str] = None,
        region: typing.Optional[Region] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProviderLinkResponse:
        """
        This connects auth providers that are password based.

        Parameters
        ----------
        provider : PasswordProviders

        username : str
            Username for provider

        password : str
            Password for provider

        vital_link_token : typing.Optional[str]

        region : typing.Optional[Region]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProviderLinkResponse
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, PasswordProviders

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.connect_password_provider(
                provider=PasswordProviders.WHOOP,
                username="username",
                password="password",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/link/provider/password/{jsonable_encoder(provider)}",
            method="POST",
            json={
                "username": username,
                "password": password,
                "region": region,
            },
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProviderLinkResponse,
                    parse_obj_as(
                        type_=ProviderLinkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def complete_password_provider_mfa(
        self,
        provider: PasswordProviders,
        *,
        mfa_code: str,
        vital_link_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProviderLinkResponse:
        """
        This connects auth providers that are password based.

        Parameters
        ----------
        provider : PasswordProviders

        mfa_code : str

        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProviderLinkResponse
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, PasswordProviders

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.complete_password_provider_mfa(
                provider=PasswordProviders.WHOOP,
                mfa_code="mfa_code",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/link/provider/password/{jsonable_encoder(provider)}/complete_mfa",
            method="POST",
            json={
                "mfa_code": mfa_code,
            },
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProviderLinkResponse,
                    parse_obj_as(
                        type_=ProviderLinkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def connect_email_auth_provider(
        self,
        *,
        email: str,
        vital_link_token: typing.Optional[str] = None,
        email_provider_auth_link_provider: typing.Optional[Providers] = OMIT,
        region: typing.Optional[Region] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        This connects auth providers that are email based.

        Parameters
        ----------
        email : str

        vital_link_token : typing.Optional[str]

        email_provider_auth_link_provider : typing.Optional[Providers]

        region : typing.Optional[Region]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.connect_email_auth_provider(
                email="email",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/link/provider/email/freestyle_libre",
            method="POST",
            json={
                "email": email,
                "provider": email_provider_auth_link_provider,
                "region": region,
            },
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Optional[typing.Any],
                    parse_obj_as(
                        type_=typing.Optional[typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_all_providers(
        self, *, vital_link_token: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[SourceLink]:
        """
        GET List of all available providers given the generated link token.

        Parameters
        ----------
        vital_link_token : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SourceLink]
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.get_all_providers()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/link/providers",
            method="GET",
            headers={
                "x-vital-link-token": str(vital_link_token) if vital_link_token is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[SourceLink],
                    parse_obj_as(
                        type_=typing.List[SourceLink],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def connect_manual_provider(
        self,
        provider: ManualProviders,
        *,
        user_id: str,
        provider_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, bool]:
        """
        Parameters
        ----------
        provider : ManualProviders

        user_id : str

        provider_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, bool]
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, ManualProviders

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.connect_manual_provider(
                provider=ManualProviders.BEURER_BLE,
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/link/provider/manual/{jsonable_encoder(provider)}",
            method="POST",
            json={
                "user_id": user_id,
                "provider_id": provider_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, bool],
                    parse_obj_as(
                        type_=typing.Dict[str, bool],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def connect_demo_provider(
        self, *, user_id: str, provider: DemoProviders, request_options: typing.Optional[RequestOptions] = None
    ) -> DemoConnectionStatus:
        """
        POST Connect the given Vital user to a demo provider.

        Parameters
        ----------
        user_id : str
            Vital user ID

        provider : DemoProviders
            Demo provider. For more information, please check out our docs (https://docs.tryvital.io/wearables/providers/test_data)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DemoConnectionStatus
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, DemoProviders

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.link.connect_demo_provider(
                user_id="user_id",
                provider=DemoProviders.APPLE_HEALTH_KIT,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/link/connect/demo",
            method="POST",
            json={
                "user_id": user_id,
                "provider": provider,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    DemoConnectionStatus,
                    parse_obj_as(
                        type_=DemoConnectionStatus,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingPeakExpiratoryFlowRateChangedEventType(str, enum.Enum):
    DAILY_DATA_PEAK_EXPIRATORY_FLOW_RATE_CREATED = "daily.data.peak_expiratory_flow_rate.created"
    DAILY_DATA_PEAK_EXPIRATORY_FLOW_RATE_UPDATED = "daily.data.peak_expiratory_flow_rate.updated"

    def visit(
        self,
        daily_data_peak_expiratory_flow_rate_created: typing.Callable[[], T_Result],
        daily_data_peak_expiratory_flow_rate_updated: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingPeakExpiratoryFlowRateChangedEventType.DAILY_DATA_PEAK_EXPIRATORY_FLOW_RATE_CREATED:
            return daily_data_peak_expiratory_flow_rate_created()
        if self is ClientFacingPeakExpiratoryFlowRateChangedEventType.DAILY_DATA_PEAK_EXPIRATORY_FLOW_RATE_UPDATED:
            return daily_data_peak_expiratory_flow_rate_updated()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingStandHourSampleType(str, enum.Enum):
    STOOD = "stood"
    IDLE = "idle"

    def visit(self, stood: typing.Callable[[], T_Result], idle: typing.Callable[[], T_Result]) -> T_Result:
        if self is ClientFacingStandHourSampleType.STOOD:
            return stood()
        if self is ClientFacingStandHourSampleType.IDLE:
            return idle()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class SleepColumnExprSleep(str, enum.Enum):
    SESSION_START = "session_start"
    SESSION_END = "session_end"
    STATE = "state"
    TYPE = "type"
    DURATION_SECOND = "duration_second"
    STAGE_ASLEEP_SECOND = "stage_asleep_second"
    STAGE_AWAKE_SECOND = "stage_awake_second"
    STAGE_LIGHT_SECOND = "stage_light_second"
    STAGE_REM_SECOND = "stage_rem_second"
    STAGE_DEEP_SECOND = "stage_deep_second"
    STAGE_UNKNOWN_SECOND = "stage_unknown_second"
    LATENCY_SECOND = "latency_second"
    HEART_RATE_MINIMUM = "heart_rate_minimum"
    HEART_RATE_MEAN = "heart_rate_mean"
    HEART_RATE_MAXIMUM = "heart_rate_maximum"
    HEART_RATE_DIP = "heart_rate_dip"
    HEART_RATE_RESTING = "heart_rate_resting"
    EFFICIENCY = "efficiency"
    HRV_MEAN_RMSSD = "hrv_mean_rmssd"
    HRV_MEAN_SDNN = "hrv_mean_sdnn"
    SKIN_TEMPERATURE = "skin_temperature"
    SKIN_TEMPERATURE_DELTA = "skin_temperature_delta"
    RESPIRATORY_RATE = "respiratory_rate"
    SCORE = "score"
    SOURCE_TYPE = "source_type"
    SOURCE_PROVIDER = "source_provider"
    SOURCE_APP_ID = "source_app_id"
    TIME_ZONE = "time_zone"

    def visit(
        self,
        session_start: typing.Callable[[], T_Result],
        session_end: typing.Callable[[], T_Result],
        state: typing.Callable[[], T_Result],
        type: typing.Callable[[], T_Result],
        duration_second: typing.Callable[[], T_Result],
        stage_asleep_second: typing.Callable[[], T_Result],
        stage_awake_second: typing.Callable[[], T_Result],
        stage_light_second: typing.Callable[[], T_Result],
        stage_rem_second: typing.Callable[[], T_Result],
        stage_deep_second: typing.Callable[[], T_Result],
        stage_unknown_second: typing.Callable[[], T_Result],
        latency_second: typing.Callable[[], T_Result],
        heart_rate_minimum: typing.Callable[[], T_Result],
        heart_rate_mean: typing.Callable[[], T_Result],
        heart_rate_maximum: typing.Callable[[], T_Result],
        heart_rate_dip: typing.Callable[[], T_Result],
        heart_rate_resting: typing.Callable[[], T_Result],
        efficiency: typing.Callable[[], T_Result],
        hrv_mean_rmssd: typing.Callable[[], T_Result],
        hrv_mean_sdnn: typing.Callable[[], T_Result],
        skin_temperature: typing.Callable[[], T_Result],
        skin_temperature_delta: typing.Callable[[], T_Result],
        respiratory_rate: typing.Callable[[], T_Result],
        score: typing.Callable[[], T_Result],
        source_type: typing.Callable[[], T_Result],
        source_provider: typing.Callable[[], T_Result],
        source_app_id: typing.Callable[[], T_Result],
        time_zone: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is SleepColumnExprSleep.SESSION_START:
            return session_start()
        if self is SleepColumnExprSleep.SESSION_END:
            return session_end()
        if self is SleepColumnExprSleep.STATE:
            return state()
        if self is SleepColumnExprSleep.TYPE:
            return type()
        if self is SleepColumnExprSleep.DURATION_SECOND:
            return duration_second()
        if self is SleepColumnExprSleep.STAGE_ASLEEP_SECOND:
            return stage_asleep_second()
        if self is SleepColumnExprSleep.STAGE_AWAKE_SECOND:
            return stage_awake_second()
        if self is SleepColumnExprSleep.STAGE_LIGHT_SECOND:
            return stage_light_second()
        if self is SleepColumnExprSleep.STAGE_REM_SECOND:
            return stage_rem_second()
        if self is SleepColumnExprSleep.STAGE_DEEP_SECOND:
            return stage_deep_second()
        if self is SleepColumnExprSleep.STAGE_UNKNOWN_SECOND:
            return stage_unknown_second()
        if self is SleepColumnExprSleep.LATENCY_SECOND:
            return latency_second()
        if self is SleepColumnExprSleep.HEART_RATE_MINIMUM:
            return heart_rate_minimum()
        if self is SleepColumnExprSleep.HEART_RATE_MEAN:
            return heart_rate_mean()
        if self is SleepColumnExprSleep.HEART_RATE_MAXIMUM:
            return heart_rate_maximum()
        if self is SleepColumnExprSleep.HEART_RATE_DIP:
            return heart_rate_dip()
        if self is SleepColumnExprSleep.HEART_RATE_RESTING:
            return heart_rate_resting()
        if self is SleepColumnExprSleep.EFFICIENCY:
            return efficiency()
        if self is SleepColumnExprSleep.HRV_MEAN_RMSSD:
            return hrv_mean_rmssd()
        if self is SleepColumnExprSleep.HRV_MEAN_SDNN:
            return hrv_mean_sdnn()
        if self is SleepColumnExprSleep.SKIN_TEMPERATURE:
            return skin_temperature()
        if self is SleepColumnExprSleep.SKIN_TEMPERATURE_DELTA:
            return skin_temperature_delta()
        if self is SleepColumnExprSleep.RESPIRATORY_RATE:
            return respiratory_rate()
        if self is SleepColumnExprSleep.SCORE:
            return score()
        if self is SleepColumnExprSleep.SOURCE_TYPE:
            return source_type()
        if self is SleepColumnExprSleep.SOURCE_PROVIDER:
            return source_provider()
        if self is SleepColumnExprSleep.SOURCE_APP_ID:
            return source_app_id()
        if self is SleepColumnExprSleep.TIME_ZONE:
            return time_zone()
